@@ -11,15 +11,16 @@ use warnings;
 use Exporter 'import';
 
 use warnings;
+use File::Basename;
 use MIME::Base64;
 use Lemonldap::NG::Common::CGI;
 use CGI::Cookie;
 require POSIX;
 use Lemonldap::NG::Portal::_i18n;      #inherits
-use Lemonldap::NG::Common::Safelib;    #link protected safe Safe object
 use Lemonldap::NG::Common::Apache::Session
   ;    #link protected session Apache::Session object
-use Safe;
+use Lemonldap::NG::Common::Safe;    #link protected safe Safe object
+use Lemonldap::NG::Common::Safelib;
 use Digest::MD5;
 
 # Special comments for doxygen
@@ -62,7 +63,7 @@ use Digest::MD5;
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '0.992';
+our $VERSION = '1.0.2';
 
 use base qw(Lemonldap::NG::Common::CGI Exporter);
 our @ISA;
@@ -187,9 +188,15 @@ our %EXPORT_TAGS = ( 'all' => [ @EXPORT, 'import' ], );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
-# Secure jail
+# Share secure jail between threads
 our $safe;
-our $self;    # Safe cannot share a variable declared with my
+
+BEGIN {
+    eval {
+        require threads::shared;
+        threads::shared::share($safe);
+    };
+}
 
 ##@cmethod Lemonldap::NG::Portal::Simple new(hashRef args)
 # Class constructor.
@@ -218,10 +225,10 @@ sub new {
         "You've to indicate a an Apache::Session storage module !" )
       unless ( $self->{globalStorage} );
     unless ( $self->{globalStorage}->can('populate') ) {
-    eval "require " . $self->{globalStorage};
-    $self->abort( "Configuration error",
-        "Module " . $self->{globalStorage} . " not found in \@INC" )
-      if ($@);
+        eval "require " . $self->{globalStorage};
+        $self->abort( "Configuration error",
+            "Module " . $self->{globalStorage} . " not found in \@INC" )
+          if ($@);
     }
 
     # Use LemonLDAP::NG custom Apache::Session
@@ -235,10 +242,10 @@ sub new {
     foreach my $otherStorage ( $self->{samlStorage}, $self->{casStorage} ) {
         if ( $otherStorage ne $self->{globalStorage} ) {
             eval "require " . $otherStorage;
-        $self->abort( "Configuration error",
+            $self->abort( "Configuration error",
                 "Module " . $otherStorage . " not found in \@INC" )
-          if ($@);
-    }
+              if ($@);
+        }
     }
 
     # 2. Domain
@@ -473,7 +480,7 @@ sub setDefaultValues {
       unless ( defined( $self->{portalDisplayChangePassword} ) );
     $self->{portalDisplayAppslist} = 1
       unless ( defined( $self->{portalDisplayAppslist} ) );
-    $self->{portalAutocomplete} ||= "off";
+    $self->{portalAutocomplete} ||= 0;
     $self->{portalRequireOldPassword} = 1
       unless ( defined( $self->{portalRequireOldPassword} ) );
     $self->{portalOpenLinkInNewWindow} = 0
@@ -539,7 +546,7 @@ sub setDefaultValues {
 
     # SAML
     $self->{samlIdPResolveCookie} ||= $self->{cookieName} . "idp";
-    $self->{samlStorage}          ||= $self->{globalStorage};
+    $self->{samlStorage} ||= $self->{globalStorage};
     if ( !$self->{samlStorageOptions} or !%{ $self->{samlStorageOptions} } ) {
         $self->{samlStorageOptions} = $self->{globalStorageOptions};
     }
@@ -555,7 +562,7 @@ sub setDefaultValues {
       unless defined $self->{samlAuthnContextMapKerberos};
 
     # CAS
-    $self->{casStorage}        ||= $self->{globalStorage};
+    $self->{casStorage} ||= $self->{globalStorage};
     if ( !$self->{casStorageOptions} or !%{ $self->{casStorageOptions} } ) {
         $self->{casStorageOptions} = $self->{globalStorageOptions};
     }
@@ -573,6 +580,7 @@ sub setDefaultValues {
 
     # Other
     $self->{logoutServices} ||= {};
+    $self->{useSafeJail} = 1 unless defined $self->{useSafeJail};
 
 }
 
@@ -806,13 +814,21 @@ sub getApacheSession {
     return \%h;
 }
 
+##@method string getApacheHtdocsPath()
+# Return the absolute path to the htdocs directory where is portal script.
+# @return path string
+sub getApacheHtdocsPath {
+  my $self = shift;
+  my ( $name, $path, $suffix ) = fileparse( $ENV{SCRIPT_FILENAME} );
+  return $path;
+}
+
 ## @method protected string _md5hash(string s)
 # Return md5(s)
 # @param $s String to hash
 # @return hashed value
 sub _md5hash {
     my ( $self, $s ) = splice @_;
-    $s ||= '';
     return substr( Digest::MD5::md5_hex($s), 0, 32 );
 }
 
@@ -1075,7 +1091,10 @@ sub get_module {
 sub safe {
     my $self = shift;
     return $safe if ($safe);
-    $safe = new Safe;
+
+    $safe = Lemonldap::NG::Common::Safe->new($self);
+
+    # Get custom functions
     my @t =
       $self->{customFunctions}
       ? split( /\s+/, $self->{customFunctions} )
@@ -1094,10 +1113,17 @@ sub safe {
             }";
         $self->lmLog( $@, 'error' ) if ($@);
     }
+
+    # Share %ENV
     $safe->share_from( 'main', ['%ENV'] );
+
+    # Share Safelib
     $safe->share_from( 'Lemonldap::NG::Common::Safelib',
         $Lemonldap::NG::Common::Safelib::functions );
+
+    # Share custom functions and &encode_base64
     $safe->share( '&encode_base64', @t );
+
     return $safe;
 }
 
@@ -1524,6 +1550,7 @@ sub controlExistingSession {
                 $self->_subProcess(qw(autoPost)) if ( $self->{postUrl} );
 
                 # Display logout message
+                untie %$h;
                 return PE_LOGOUT_OK;
             }
 
@@ -1534,6 +1561,13 @@ sub controlExistingSession {
                 $self->_sub( 'removeOther', $id );
             }
             untie %$h;
+
+            # Special ajax request "ping" to check if session is available
+            if ( $self->param('ping') ) {
+                print $self->header( -type => 'application/json' )
+                  . '{"auth":true}';
+                $self->quit();
+            }
             $self->{id} = $id;
 
             # A session has been found => call existingSession
@@ -1552,6 +1586,10 @@ sub controlExistingSession {
 
     # Display logout success if logout asked
     # and we do not have valid session
+    if ( $self->param('ping') ) {
+        print $self->header( -type => 'application/json' ) . '{"auth":false}';
+        $self->quit();
+    }
     return PE_LOGOUT_OK if $self->param('logout');
 
     # Else continue authentication process
@@ -1708,15 +1746,13 @@ sub setSessionInfo {
     PE_OK;
 }
 
-##@apmethod int setMacro()
+##@apmethod int setMacros()
 # Macro mechanism.
 # * store macro results in $self->{sessionInfo}
 #@return Lemonldap::NG::Portal constant
 sub setMacros {
-    local $self = shift;
-    $self->safe->share('$self');
+    my $self = shift;
     while ( my ( $n, $e ) = each( %{ $self->{macros} } ) ) {
-        $e =~ s/\$(?!ENV)(\w+)/\$self->{sessionInfo}->{$1}/g;
         $self->{sessionInfo}->{$n} = $self->safe->reval($e);
     }
     PE_OK;
@@ -1727,11 +1763,9 @@ sub setMacros {
 # * store all groups name that the user match in $self->{sessionInfo}->{groups}
 #@return Lemonldap::NG::Portal constant
 sub setLocalGroups {
-    local $self = shift;
+    my $self = shift;
     my $groups;
-    $self->safe->share('$self');
     while ( my ( $group, $expr ) = each %{ $self->{groups} } ) {
-        $expr =~ s/\$(\w+)/\$self->{sessionInfo}->{$1}/g;
         $groups .= $group . $self->{multiValuesSeparator}
           if ( $self->safe->reval($expr) );
     }
@@ -1749,6 +1783,8 @@ sub setPersistentSessionInfo {
 
     # Do not restore infos if session already opened
     unless ( $self->{id} ) {
+        my $key = $self->{sessionInfo}->{ $self->{whatToTrace} };
+        return PE_OK unless ( $key and length($key) );
         my $h =
           $self->getApacheSession(
             $self->_md5hash( $self->{sessionInfo}->{ $self->{whatToTrace} } ),
@@ -1907,7 +1943,6 @@ sub grantSession {
 
     # Eval grantSessionRule
     my $grantSessionRule = $self->{grantSessionRule};
-    $grantSessionRule =~ s/\$(\w+)/\$self->{sessionInfo}->{$1}/g;
 
     unless ( $self->safe->reval($grantSessionRule) ) {
         $self->lmLog(
@@ -2040,7 +2075,6 @@ sub issuerForAuthUser {
     my $rule = $self->{ 'issuerDB' . $issuerDBtype . 'Rule' };
 
     if ( defined $rule ) {
-        $rule =~ s/\$(\w+)/\$self->{sessionInfo}->{$1}/g;
 
         $self->lmLog( "Applying rule: $rule", 'debug' );
 
@@ -2576,7 +2610,7 @@ find the user distinguished name (dn) was refused by the server
 =head1 SEE ALSO
 
 L<Lemonldap::NG::Handler>, L<Lemonldap::NG::Portal::SharedConf>, L<CGI>,
-http://wiki.lemonldap.objectweb.org/xwiki/bin/view/NG/Presentation
+L<http://lemonldap-ng.org/>
 
 =head1 AUTHOR
 
@@ -2585,7 +2619,7 @@ Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://forge.objectweb.org/tracker/?group_id=274>
+L<http://jira.ow2.org>
 
 =head1 DOWNLOAD
 
@@ -2598,7 +2632,7 @@ Copyright (C) 2005, 2009, 2010 by Xavier Guimard E<lt>x.guimard@free.frE<gt> and
 Clement Oudot, E<lt>coudot@linagora.comE<gt>
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.4 or,
+it under the same terms as Perl itself, either Perl version 5.10.0 or,
 at your option, any later version of Perl 5 you may have available.
 
 =cut
